# specifically chose not to use *args here cuz that unnecessarily unwinds generator expressions
def first_not_none(iterable, fallback=None):
    """ return the first item that is not None """
    for obj in iterable:
        if obj is not None: return obj
    return fallback

# specifically chose not to use *args here cuz that unnecessarily unwinds generator expressions
def first_truthy(iterable, fallback=None):
    """ return the first item that is truth """
    for obj in iterable:
        if obj: return obj
    return fallback

def sort(obj, *args, **kwargs):
    """ a sort that actually returns the object being sorted """
    obj.sort(*args, **kwargs)
    return obj





import unittest
from .. import property

class PropertyTest(unittest.TestCase):

    def setUp(self): pass
    def tearDown(self): pass

    def test_cached_property(self):

        class X(object):
            def __init__(self):
                super(X, self).__init__()
                self.calculation_count = 0
            @property.cached_property
            def expensive_value(self):
                self.calculation_count += 1
                return True

        x = X()
        self.assertEquals(0, x.calculation_count)
        self.assertTrue(x.expensive_value)
        self.assertEquals(1, x.calculation_count)
        self.assertTrue(x.expensive_value)
        self.assertEquals(1, x.calculation_count)
        del x.expensive_value
        self.assertTrue(x.expensive_value)
        self.assertEquals(2, x.calculation_count)
        x.expensive_value = False
        self.assertFalse(x.expensive_value)
        self.assertEquals(2, x.calculation_count)

    def test_dynamic_cached_property(self):
        class X(object):
            def __init__(self):
                super(X, self).__init__()

            @classmethod
            def create_caching(kls):
                @property.cached_property
                def _tmp(self):
                    print "bye"
                kls.yox = _tmp

        def _tmp(self):
            print "hello"
            return 1

        #X.yo = _tmp
        #property.cached_property(X.yo)
        #X.blah = property.cached_property
        #X.blah(_tmp,attr='yo')
#        X.blah(X.yo)
        X.yo = property.cached_property(_tmp)
        x = X()
        from pprint import pprint
        pprint(x.__dict__)
        print x.yo
        pprint(x.__dict__)
        print x.yo
        print x.yo
#        x._tmp
        #X.create_caching()
        #x = X()
        #x.yox()
        #x.yox()
        #x.yox()


    #def test_descriptor(self):
        #class Descriptor(self,f)



from .list import first_not_none
import sys

# allows wrapping of another error or not-- a good class to use as your base for your own exception tree
class Error(Exception):
    def __init__(self, msg=None, err=None):
        super(Error, self).__init__(msg or err and unicode(err) or '')
        self.msg = msg or ''
        self.err = err

    @property
    def wrapped_error_str(self):
        if not self.err: return ''
        return u'[%s:%s]' % (self.err.__class__.__name__, self.err)

    def __str__(self): return unicode(self).encode('utf-8')
    def __unicode__(self):
        return u'%s%s' % (self.msg, self.wrapped_error_str)
    def __repr__(self):
        return '%s(%s,%s)' % (self.__class__.__name__, repr(self.msg), repr(self.err))

    # raises wrapped exception appropriately or normal excpetion normally
    def _raise(self):
        if self.err:
            raise self, None, sys.exc_info()[2]
        raise self




def merge(destination, *sources, **kwargs):
    if isinstance(sources, dict): sources = [sources]
    for source in sources:
        if source: destination.update(source)
    for k,v in kwargs.iteritems():
        destination[k]=v
    return destination

def mpop(d, *args, **kwargs):
    for a in args:
        if d.get(a) is not None: return d.pop(a)
    return kwargs.get('fallback',None)

def mget(d, *args, **kwargs):
    for a in args:
        if d.get(a) is not None: return d[a]
    return kwargs.get('fallback',None)

def getdict(d, *only_args):
    return dict((k,v)for k,v in d.iteritems() if k in only_args)

def popdict(d, *only_args):
    subdict = getdict(d, *only_args)
    for k in subdict: d.pop(k)
    return subdict

def kwargs_str(d, *only_args):
    return ','.join(["%s=%s"%(k,repr(d[k])) for k in (only_args or d.keys())])




def nstrip(s): return None if s is None else s.strip()
def nlower(s): return None if s is None else s.lower()
def nupper(s): return None if s is None else s.upper()

def cutoff(s, amt): return s if len(s or '') <= amt else "%s..."%s[0:amt-3]



import unittest
from ..exception import Error

class DomainError(Error): pass

class ErrorTest(unittest.TestCase):

    def setUp(self): pass
    def tearDown(self): pass

    def test_representations(self):
        literal = u"blah \u2603"
        err = DomainError(literal)
        self.assertEquals(literal, unicode(err))
        self.assertEquals(unicode(literal).encode('utf-8'), str(err))
        self.assertEquals('DomainError(%s,None)'%repr(literal), repr(err))

        err = DomainError("domain "+literal, ValueError(literal))
        self.assertEquals("domain %s[ValueError:%s]"%(literal,literal), unicode(err))

    def test_raise(self):
        with self.assertRaises(DomainError):
            DomainError()._raise()

        with self.assertRaises(DomainError):
            DomainError("outside", ValueError("inside"))._raise()

    def test_everything(self):
        in_appropriate_except_clause = False
        try:
            try:
                raise ValueError(u"inner msg")
            except ValueError as err:
                DomainError(u"outer msg", err)._raise()
        except DomainError as err:
            in_appropriate_except_clause = True
            self.assertEquals(ValueError, err.err.__class__)
            self.assertEquals(u"inner msg", unicode(err.err))
            self.assertEquals(u"outer msg", err.msg)
        self.assertTrue(in_appropriate_except_clause)



    # this test needs human visual inspection, so it is not run by default -- unskip to force a run
    # haven't spent time to figure out how to do programmatically -- just checking that raising a wrapped excpetion also includes the stack trace from that exception
    @unittest.skip
    def test_good_stacktrace(self):
        def bar(): raise ValueError(u"inner msg")
        def foo(): return bar()
        def baz(err): DomainError(u"outer msg", err)._raise()

        try:
            foo()
        except ValueError as err:
            baz(err)

    # this just confirms what happens when you don't do the appropriate raise
    @unittest.skip
    def test_bad_stacktrace(self):
        def bar(): raise ValueError(u"inner msg")
        def foo(): return bar()
        def baz(err): raise DomainError(u"outer msg", err)

        try:
            foo()
        except ValueError as err:
            baz(err)



import unittest
from ..dict import merge,mpop,mget

class DictTest(unittest.TestCase):

    def setUp(self): pass
    def tearDown(self): pass

    def test_merge(self):
        x = {'x':1}
        y = {'y':2}
        z = {'z':3}
        q = {'y':4}
        self.assertEquals({'x':1}, merge({},x))
        self.assertEquals({'x':1,'y':2,'z':3}, merge({},x,y,z))
        self.assertEquals({'x':1},x)
        self.assertEquals({'y':2},y)
        self.assertEquals({'z':3},z)
        self.assertEquals({'x':1,'y':4,'z':3}, merge({},x,y,z,q))
        self.assertEquals({'x':1},x)
        self.assertEquals({'y':2},y)
        self.assertEquals({'z':3},z)
        self.assertEquals({'x':1,'y':4,'z':3}, merge(x,y,z,q))
        self.assertEquals({'x':1,'y':4,'z':3},x)
        self.assertEquals({'y':2},y)
        self.assertEquals({'z':3},z)

    def test_mget(self):
        d = {'x':None, 'y':0}
        self.assertEquals(0,mget(d,'x','y'))
        self.assertEquals({'x':None, 'y':0},d)
        self.assertIsNone(mget(d,'z','q'))
        self.assertEquals(-1,mget(d,'z','q',fallback=-1))
        self.assertEquals({'x':None, 'y':0},d)

    def test_mpop(self):
        d = {'x':None, 'y':0}
        self.assertEquals(0, mpop(d,'x','y'))
        self.assertEquals({'x':None}, d)
        self.assertIsNone(mpop(d,'z','q'))
        self.assertEquals(-1, mpop(d,'z','q',fallback=-1))
        self.assertEquals({'x':None}, d)
       




from .property import is_cached

def kwargs_str(obj, attrs=None, cached=None):
    ktuples = [(k,repr(getattr(obj,k))) for k in (attrs or []) if hasattr(obj,k)]
    ktuples += [(k,repr(getattr(obj,k))) for k in (cached or []) if is_cached(obj,k)]
    return ','.join(['%s=%s'%t for t in ktuples])

def from_obj(obj, obj_attrs=None, *attrs):
    d = dict((a,getattr(obj,a)) for a in attrs if obj.hasattr(obj,a))
    for a in obj_attrs: 
        if obj.hasattr(a):
            d.update(from_obj(getattr(obj,a)))
    return d

def mgetattr(obj, attrs, *args): return tuple([getattr(obj, a, *args) for a in attrs])
    
def nchain(obj, *attrs):
    val = obj
    for a in attrs:
        if val is None: return None
        val = getattr(val,a,None)
    return val

def ensureattr(obj, attr, val):
    if not hasattr(obj,attr): setattr(obj,attr,val)
    return getattr(obj,attr)



CACHED_PREFIX = '_cached_'

class cached_property(object):
    def __init__(self, f, name=None, *fargs):
        super(cached_property, self).__init__()
        self.f = f
        self.fargs = fargs
        self.attr = '%s%s' % (CACHED_PREFIX, name or f.__name__)

    def __get__(self, obj, type=None):
        if not hasattr(obj, self.attr):
            setattr(obj, self.attr, self.f(obj, *self.fargs))
        return getattr(obj, self.attr)

    def __set__(self, obj, value):
        setattr(obj, self.attr, value)

    def __delete__(self, obj):
        if hasattr(obj, self.attr):
            delattr(obj, self.attr)

# so you don't have to remember what variables are stored as, and you have a way to see if caching happened without triggering the actual evaluation
def is_cached(obj, attr): return hasattr(obj, cached_key(attr))
def cached_value(obj, attr, default=None): return getattr(obj, cached_key(attr), default)
def cached_key(attr): return '%s%s' % (CACHED_PREFIX, attr)
def delete_cache(obj, *attrs): 
    for attr in attrs: 
        if is_cached(obj,attr): 
            delattr(obj, cached_key(attr))


# for backward compatability
lazy_property = cached_property



import unittest
from ..list import first_not_none

class ListTest(unittest.TestCase):

    def setUp(self): pass
    def tearDown(self): pass

    def test_first_not_none(self):
        self.assertEquals(3,first_not_none((None,None,3)))
        self.assertEquals(3,first_not_none((None,None,3),4))
        self.assertIsNone(first_not_none((None,None)))
        self.assertEquals(4,first_not_none((None,None),4))
        self.assertEquals(4,first_not_none((None,None),4))
        self.assertEquals(5,first_not_none((5,4,3)))

        #generators
        self.assertIsNone(first_not_none((None for i in xrange(100))))
        self.assertEquals(-1,first_not_none((None for i in xrange(100)),-1))
        self.assertEquals(-1,first_not_none((i>=99 and -1 or None for i in xrange(100))))





#!/usr/bin/env python
from distutils.core import setup

VERSION = '0.8.1'

setup(
    name='utilspy',
    version=VERSION,
    description='Random useful python utilities that don\'t seem to exist elsewhere',
    long_description = open('README.rst').read(),
    author='Michael Prior',
    author_email='prior@cracklabs.com',
    url='https://github.com/prior/utilspy',
    download_url='https://github.com/prior/utilspy/tarball/v%s'%VERSION,
    license='LICENSE.txt',
    packages=['utils','utils.test'],
    install_requires=[ 'nose==1.1.2' ]
)
